/*
    FEARLUS/SPOM 1-1-4: AbstractReport.h
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/* AbstractReport
 *
 * This is a class for other reports to subclass from, saving duplication of
 * code from the design in earlier versions of the software.
 */

#import "FearlusThing.h"
#import "Report.h"

@class ModelSwarm, Parameter, AssocArray;

@interface AbstractReport: FearlusThing <Report> {
  Parameter *parameter;
  ModelSwarm *model;
  id <List> fixedYears;
  id <List> repeatYears;
}

+create: aZone;	// Subclasses may override this method
-(void)setModelSwarm: (ModelSwarm *)m andParameters: (Parameter *)p;
				// Subclasses should not override this
				// method (but some might--and may
				// rely on it being called AFTER
				// options have been set)
-(void)reportForYear: (unsigned)year toFile: (FILE *)fp;
				// Subclasses must override this method
-(BOOL)setOption: (char *)option toValue: (char *)value;
				// Subclasses may override this method
-(BOOL)setOptionFlag: (char *)option toValue: (BOOL)value;
				// Subclasses may override this method
-(void)setFixedYears: (id <List>)y;
				// Subclasses should not override this method
-(void)setRepeatYears: (id <List>)y;
				// Subclasses should not override this method
-(BOOL)reportingForYear: (unsigned)year;
				// Subclasses should not override this method
				// (but some might)

@end
