/*
    FEARLUS/SPOM 1-1-4: AbstractSocialLandManager.h
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

/*
 * Interface for the AbstractSocialLandManager class. This contains
 * items in common to all social Land Manager classes. Social land
 * managers are those who give or receive social approval.
 */

#import "AbstractLandManager.h"
#import <collections/List.h>

@class AssocArray;

@interface AbstractSocialLandManager: AbstractLandManager {
  id <List> triggers;
  id <List> approvers;
  id <List> disapprovers;
  id <List> approved;
  id <List> disapproved;
  id <List> neutral;
  AssocArray *approver_amt;
  AssocArray *disapprover_amt;
  double approval;		// received
  double disapproval;		// received
  double approve;		// given
  double disapprove;		// given
}

+(Class)getSubPopClass;
-(void)initialiseWithEnvironment: (Environment *)e
		   landAllocator: (LandAllocator *)la
			  colour: (int)col;

-acceptDisapproval: (double)dis from: (AbstractSocialLandManager *)disapprover;
-acceptApproval: (double)app from: (AbstractSocialLandManager *)approver;
-(void)prepareApprovals;	// Called from schedule, and is also used for
				// disapprovals. This method is used to make
				// any preparations required for this year's
				// round of approvals and disapprovals (e.g.
				// setting things to zero).
-(void)determineApprovals;	// Called from schedule, and is also used for
				// disapprovals. Subclasses probably won't
				// need to override this method.
-(double)getApproval;
-(double)getDisapproval;
-(double)getApprove;
-(double)getDisapprove;
-(double)getApprovalFrom: (AbstractSocialLandManager *)lm;
-(BOOL)approvedBy: (AbstractSocialLandManager *)lm;
-(double)getDisapprovalFrom: (AbstractSocialLandManager *)lm;
-(BOOL)disapprovedBy: (AbstractSocialLandManager *)lm;
-(id <List>)getApprovers;
-(id <List>)getDisapprovers;
-(id <List>)getApproved;
-(id <List>)getDisapproved;
-(id <List>)getNeutral;
-(void)approve: (double)amt of: (AbstractSocialLandManager *)lm;
-(void)disapprove: (double)amt of: (AbstractSocialLandManager *)lm;
-(void)neitherApproveNorDisapproveOf: (AbstractSocialLandManager *)lm;
-(void)drop;

@end
