/*
    FEARLUS/SPOM 1-1-4: AllLowPollutionTrigger.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

/* Implementation of AllLowPollutionTrigger class
 */

#import "AllLowPollutionTrigger.h"
#import "FearlusOutput.h"
#import "AbstractSocialLandManager.h"
#import "LandParcel.h"
#import "LandUse.h"
#import "MiscFunc.h"

@implementation AllLowPollutionTrigger

/* load:
 *
 * Load in the parameter for this trigger
 */

-(void)load: (FILE *)fp {
  if(fscanf(fp, " PollutionThreshold: %lf", &pollution_threshold) != 1) {
    fprintf(stderr, "Format error in trigger file\n");
    [MiscFunc fileHere: fp];
    abort();
  }
  [super load: fp];
}

/* writeParameters:
 *
 * Write out the parameter for this trigger
 */

-(void)writeParameters: (FILE *)fp {
  [super writeParameters: fp];
  fprintf(fp, "Pollution threshold\t%g%s", pollution_threshold,
	  [FearlusOutput nl]);
}

/* manager:ofManager:approves:disapproves:
 *
 * Determine whether all of the object land manager's land parcels
 * have pollution less than or equal to the threshold. 
 */

-(void)manager: (AbstractSocialLandManager *)lm_subj
     ofManager: (AbstractSocialLandManager *)lm_obj
      approves: (double *)app
   disapproves: (double *)disapp {
  id ix;
  LandParcel *lp;

  (*app) = approval;
  for(ix = [[lm_obj getLandParcels] begin: scratchZone],
	lp = (LandParcel *)[ix next];
      [ix getLoc] == Member;
      lp = (LandParcel *)[ix next]) {
    if([[lp getLandUse] getPollution] > pollution_threshold) {
      (*app) = 0.0;
      break;
    }
  }
  [ix drop];
}

@end
