/*
    FEARLUS/SPOM 1-1-4: CBRSocialLandManager.h
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

/* Interface for the CBRSocialLandManager class. This is a social land
 * manager that makes land use decisions using Case Based Reasoning,
 * trading off between two dimensions of utility: Profit and Social
 * Approval.
 */

#import <collections/List.h>
#import "AbstractSocialLandManager.h"

@class CBRCaseBase, CBROutcome, Environment, LandAllocator, LandParcel,
  LandUse, CBRCase, CBRState;

@interface CBRSocialLandManager: AbstractSocialLandManager {
  CBRCaseBase *cb;		// The case base
  id <List> events;		// A list of events that cause adjustment to
				// the salience of profit or approval
  double profit_wgt;		// The utility weight applied to profit
  double approval_wgt;		// The utility weight applied to approval
  double profit_min;		// The minimum profit utility weight value
  double approval_min;		// The minimum approval utility weight value
  double salience_margin;	// The room for adjustment of utility weights
				// between profit and approval
  double salience_adjust;	// The amount to modify the salience by for
				// each event
  double financial_aspiration;	// The aspiration threshold for money
  double social_aspiration;	// The aspiration for social approval
}

+(Class)getSubPopClass;
-(void)drop;

-(double)getFinancialAspiration;
-(double)getSocialAspiration;
-(double)getSalienceMargin;
-(double)getProfitWeightMin;
-(double)getProfitWeight;
-(double)getApprovalWeightMin;
-(double)getApprovalWeight;
-(double)getProfitApprovalWeightRatio;
-(double)getUtility;
-(double)estimateUtilityForOutcome: (CBROutcome *)outcome;

-incProfitSalience;
-decProfitSalience;
-incApprovalSalience;
-decApprovalSalience;

/* Methods overridden from AbstractLandManager */

-(void)initialiseWithEnvironment: (Environment *)e
		   landAllocator: (LandAllocator *)la
			  colour: (int)col;
-(void)allocateLandUses;
-(void)allocateInitialLandUses;
-(LandUse *)decideLandParcel: (LandParcel *)lp;
-(CBRCase *)findCaseForDecision: (LandUse *)lu state: (CBRState *)state;
-(LandUse *)caseBasedReasonLandParcel: (LandParcel *)lp;
-(void)learn;

@end
