/*
    FEARLUS/SPOM 1-1-4: CBRState.h
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

/* This is a class to store a state of the world. For now, this
 * consists of Biophysical Properties, Climate and Economy. In future
 * this could be made more general.
 */

#import "FearlusThing.h"
#import <stdio.h>
#import "CBRSimilarity.h"

@class LandParcel, LTGroupState, FearlusStream;

@interface CBRState: FearlusThing {
  LandParcel *biophys;
  LTGroupState *climate;
  LTGroupState *economy;
}

+create: (id <Zone>)z biophys: (LandParcel *)b
                      climate: (LTGroupState *)c 
                      economy: (LTGroupState *)e;
-clone: (id <Zone>)z;
-(BOOL)equalState: (CBRState *)state;
-(CBRSimilarity *)comparedWith: cmp relativeTo: base;
-(LandParcel *)getBiophys;
-(LTGroupState *)getClimate;
-(LTGroupState *)getEconomy;
-(void)printToFile: (FILE *)fp;
-(void)printStream: (FearlusStream *)stream;
-print;
-(void)drop;

@end
