/*
    FEARLUS/SPOM 1-1-4: Fatal.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

#import "Fatal.h"
//#import <stdlib.h>
#import <errno.h>
#import <stdio.h>
#import <string.h>
#import <misc.h>

@implementation Fatal

/* +die
 *
 * Exit the application. For now, just abort, but in future could shut down
 * nicely.
 */

+(void)die {
  abort();
}

/* +invalidSeed:
 *
 * An invalid seed has been supplied.
 */

+(void)invalidSeed: (const char *)arg {
  fprintf(stderr, "Invalid seed: %s\n", arg);
  [self die];
}

/* +noSuchClass:
 *
 * A class has been asked for that does not exist.
 */

+(void)noSuchClass: (const char *)arg {
  fprintf(stderr, "Invalid class: %s\n", arg);
  [self die];
}

/* +invalidRNGClass:
 *
 * A class has been asked for that is not a valid RNG.
 */

+(void)invalidRNGClass: (const char *)arg {
  fprintf(stderr, "FATAL: Not a valid random number generator class: %s\n",
	  arg);
  [self die];
}

/* +readingFile:errno:
 *
 * An error occurred whilst opening a file for reading.
 */

+(void)readingFile: (const char *)filename err: (int)err {
  fprintf(stderr, "FATAL: Error opening file %s for reading: %s\n", filename,
	  strerror(err));
  [self die];
}

/* +batchNoParameterFile
 *
 * The user has started the program in batch mode without specifying a 
 * parameter file.
 */

+(void)batchNoParameterFile {
  fprintf(stderr, "FATAL: No parameter file specified in batch mode\n");
  [self die];
}

@end
