/*
    FEARLUS/SPOM 1-1-4: FearlusOutput.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

/* Implementation of FearlusOutput */

#import "FearlusOutput.h"
#import "FearlusStream.h"

#import <misc.h>

/* class variables */

static FearlusStream *stream = nil;

@implementation FearlusOutput

/* nl -> newline character sequence
 *
 * Return a character sequence to print a new line on this host. On Unix
 * machines this is "\n", but on PCs, it is "\r\n".
 */

+(const char *)nl {
#ifdef __CYGWIN__
  return "\r\n";
#else
  return "\n";
#endif
}

/* setStream: -> Old Stream (or nil)
 *
 * Set the stream for this output channel.
 */

+(FearlusStream *)setStream: (FearlusStream *)s {
  FearlusStream *old = stream;

  stream = s;

  return old;
}

/* getStream -> Current stream
 *
 * Get the stream for this output channel. This is assumed to be used in
 * subclasses so that they can access the stream. It is therefore a fatal
 * error to call this method when the stream is nil.
 */

+(FearlusStream *)getStream {
  if(stream == nil) {
    fprintf(stderr, "FATAL: [FearlusOutput getStream] call when stream is "
	    "nil\n");
    abort();
  }
  return stream;
}

@end
