/*
    FEARLUS/SPOM 1-1-4: HabitStrategy.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Implementation for the HabitStrategy object. 

*/

#import "HabitStrategy.h"
#import "SelectUseBucket.h"
#import "Parameter.h"
#import "Debug.h"
#import "LandParcel.h"
#import "LandUse.h"

@implementation HabitStrategy

/*

create:withManager:andParameters:

Create the strategy, setting the manager and parameters.

*/

+(id <Strategy>)create: aZone
	   withManager: (id <StrategyManager>)lmgr
	 andParameters: (Parameter *)p {
  HabitStrategy *obj;

  obj = [super create: aZone];
  obj->lm = lmgr;
  obj->parameter = p;
  return obj;
}

/*

decideLandUseForParcel:

Decide the land use for the land parcel by simply selecting its previous land
use.

*/

-(LandUse *)decideLandUseForParcel: (LandParcel *)lp {
  [Debug verbosity: M(showDecisionAlgorithmDetail)
	 write: "Using habit imitation strategy of land manager %u to "
	 "determine land use of land parcel %u at (%d, %d) with land use %u "
	 "(%s)",
	 [lm getPIN], [lp getPIN], [lp getX],
	 [lp getY], [[lp getLandUse] getPIN], [[lp getLandUse] getLabel]];
  [lp incNImitations];
  return [lp getLandUse];
}

@end
