/*
    FEARLUS/SPOM 1-1-4: ParcelRewardingGovernment.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/* Implementation of ParcelRewardingGovernment class. As with all Government
 * classes, class variables are used to store the parameters. 
 */

#import "ParcelRewardingGovernment.h"
#import "Environment.h"
#import "LandAllocator.h"
#import "AbstractLandManager.h"
#import "Debug.h"

@implementation ParcelRewardingGovernment

/* govern
 *
 * Implement the government's policy with regard to pollution: If the
 * total pollution is less than the threshold then each land manager
 * gets a reward for each land parcel they own.
 */

-(void)govern {
  if([env getPollution] < pollutionThreshold) {
    id ix;
    AbstractLandManager *lm;

    [Debug verbosity: M(showGovernment)
	   write: "Issuing reward to all land managers as environment "
	   "pollution %g is less than threshold %g",
	   [env getPollution], pollutionThreshold];
    for(ix = [[landAllocator getLandManagers] begin: scratchZone],
	  lm = (AbstractLandManager *)[ix next];
	[ix getLoc] == Member;
	lm = (AbstractLandManager *)[ix next]) {
      double reward_amount = reward * [lm getNumberOfLandParcelsOwned];

      [Debug verbosity: M(showGovernmentDetail)
	     write: "Issuing reward %g to land manager %u", reward,
	     [lm getPIN]];
      [lm acceptReward: reward_amount];
      expenditure += reward_amount;
    }
    [ix drop];
  }
  else {
    [Debug verbosity: M(showGovernment)
	   write: "Not issuing reward because environment pollution %g is "
	   "more than or equal to threshold %g", [env getPollution],
	   pollutionThreshold];
  }
}

@end;
