/*
    FEARLUS/SPOM 1-1-4: Progress.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

#import "Progress.h"
#import "FearlusStream.h"

progress_t detail;

@implementation Progress

/* +detail:write:
 *
 * Write a progress message with the specified level of detail.
 */

+(void)detail: (progress_t)level write: (char *)format, ... {
  va_list ap;

  if(level <= detail) {
    [[self getStream] write: "Progress: "];
 
    va_start(ap, format);

    [[self getStream] vwrite: format args: ap];

    [[self getStream] write: "%s", [self nl]];

    va_end(ap);
  }

}

/* +setDetail:
 *
 * Set the maximum level of detail at which messages will be printed
 */

+(void)setDetail: (progress_t)level {
  detail = level;
}

/* +showDetail: -> Boolean
 *
 * Return whether or not progress should be shown at the level of detail
 * supplied as argument.
 */

+(BOOL)showDetail: (progress_t)level {
  return (level <= detail) ? YES : NO;
}


@end
