/*
    FEARLUS/SPOM 1-1-4: RandomSelectionStrategy.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

Implementation of the RandomSelectionStrategy class.

*/

#import "RandomSelectionStrategy.h"
#import "MiscFunc.h"
#import "Tuple.h"
#import "LandParcel.h"
#import "AssocArray.h"

@implementation RandomSelectionStrategy

/* -selectBids:fromList:
 *
 * Select the bids at random.
 */

-(void)selectBids: (AssocArray *)bids fromList: (id <List>)list {
  double total_spend = 0.0;
  id <List> buy_list = [List create: scratchZone];

  [MiscFunc shuffleList: list];

  while([list getCount] > 0) {
    LandParcel *lp = [list removeFirst];
    Tuple *bid = [bids getObjectWithKey: lp];
    double price;

    if(bid == nil) {
      fprintf(stderr, "Panic! Bid for land parcel %u at (%d, %d) not found\n",
	      [lp getPIN], [lp getX], [lp getY]);
      abort();
    }

    price = [bid getDouble];

    if(price + total_spend <= budget) {
      [buy_list addLast: lp];
      total_spend += price;
    }
  }

  while([buy_list getCount] > 0) {
    [list addLast: [buy_list removeFirst]];
  }

  [buy_list drop];
}

@end
