/*
    FEARLUS/SPOM 1-1-4: RelativeChangeTimeSeries.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/* Implementation of RelativeChangeTimeSeries
 */

#import "RelativeChangeTimeSeries.h"
#import "Number.h"
#import "Bug.h"
#import "Panic.h"
#import <math.h>

@implementation RelativeChangeTimeSeries

/* -getTimeSeries:length:
 *
 * Return the time series to use, based on the raw data obtained. The time
 * series is abs(x[t] - x[t-1]) / x[t-1], where x is the raw data
 */

-(double *)getTimeSeries: z length: (int *)len {
  int i;
  id <Index> ix;
  double *arr;
  double last = 0.0;		// Stop compiler complaints

  if(origin == nil) [Bug file: __FILE__ line: __LINE__];
				// TimeSeries not initialised

  (*len) = [time_series getCount] - 1;
  if((*len) <= 0) return NULL;

  arr = [z alloc: (*len) * sizeof(double)];

  for(ix = [time_series begin: scratchZone], [ix next], i = -1;
      [ix getLoc] == Member;
      [ix next], i++) {
    Number *num;
    double this;

    if(i >= (*len)) [Panic file: __FILE__ line: __LINE__];

    num = (Number *)[ix get];
    this = [num getDouble];

    if(i >= 0) arr[i] = fabs(this - last) / last;

    last = this;
  }
  [ix drop];

  return arr;
}

@end
