/*
    FEARLUS/SPOM 1-1-4: SPOMAbstractConnect.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
#import "SPOMAbstractConnect.h"
#import "SPOMParameter.h"
#import "Panic.h"

@implementation SPOMAbstractConnect

/* -buildObjects
 *
 * Build the dispersal object.
 */

-buildObjects {   
  // Creation of the dispersal object
  dispersal = [[SPOMParameter dispersal] create: [self getZone]];
  dispersal = [dispersal buildObjects];  
  
  return self;
}

/* -getConnect:
 *
 * Method for subclasses to implement.
 */

-(double)getConnect: (SPOMSpeciesPatch *)spat consideringHabitats: (BOOL) b {
  [Panic file: __FILE__ line: __LINE__];
				// Should be implemented by subclass
  return 0.0;
}

/* -drop
 *
 * Destroy the dispersal object
 */

-(void)drop {
  [dispersal drop];
  [super drop];
}

@end
