/*
    FEARLUS/SPOM 1-1-4: SPOMCompetitionSpecies.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
#import "SPOMCompetitionSpecies.h"
#import "SPOMParameter.h"
#import "Number.h"

@implementation SPOMCompetitionSpecies

/* -buildObjects
 *
 * Build the extra objects for SPOMCompetitionSpecies
 */

-buildObjects {
  [super buildObjects];

  outcompete = [Array create: [self getZone] setCount: [SPOMParameter nSpecies]];

  return self;
}

/* -getOutcompetetime:
 *
 * Return the amount of time taken to outcompete the given species.
 */

-(int)getOutCompetetime: (SPOMSpecies *)species {
  Number *n;

  n = [outcompete atOffset: [species getPatchArrID]];
  
  return n == nil ? 0 : [n getInt];
}

/* -setOutcompetetime:value:
 *
 * Set the amount of time taken to outcompete the given species.
 */

-setOutcompetetime: (int)arrpos value: (int)v {
  if([outcompete atOffset: arrpos] == nil) {
    [outcompete atOffset: arrpos put: [Number create: [self getZone]]];
  }

  [(Number *)[outcompete atOffset: arrpos] setInt: v];

  return self;
}

@end
