/*
    FEARLUS/SPOM 1-1-4: SPOMExtinction2.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
#import "SPOMExtinction2.h"
#import "SPOMParameter.h"
#import "SPOMSpecies.h"
#import "SPOMHabitat.h"
#import "SPOMAbstractPatch.h"
#import "SPOMSpeciesPatch.h"
#import <math.h>

@implementation SPOMExtinction2

/* -getExtinction:species:
 *
 * Compute the extinction using the formula 2
 */

-(double)getExtinction: (SPOMSpeciesPatch *)sp{  
  id ixsh;
  double H;
  SPOMHabitat *habitats;
  
  double mucoeff=1;
  double muHabSpecificCoeff=1;
    
  H = 0.0;
    
  for(ixsh = [[ [sp species] getHabitatList] begin: scratchZone],
	habitats = (SPOMHabitat *)[ixsh next];
      [ixsh getLoc] == Member;
      habitats = (SPOMHabitat *)[ixsh next]) {
    H += [ [sp patch] getValueOfHabitat: habitats];
  }
  [ixsh drop];
  
  if([SPOMParameter predation])
	mucoeff=[sp getPredationModifier];
	
  if([SPOMParameter enableHabitatSpecificMu]){
	muHabSpecificCoeff=0;
	for(ixsh = [[ [sp species] getHabitatList] begin: scratchZone],	habitats = (SPOMHabitat *)[ixsh next];
			[ixsh getLoc] == Member;
			habitats = (SPOMHabitat *)[ixsh next]) {
		muHabSpecificCoeff +=  ([ [sp patch] getValueOfHabitat: habitats] / H) * [habitats getSpeciesMuModifier : [[sp species] getID]];
	}
	[ixsh drop];
  }
	
  return (1.0 - exp(-[ [sp species] getMu] * mucoeff * muHabSpecificCoeff
		    / pow([SPOMParameter Acell] * H, [SPOMParameter xparam])));
}

@end
