/*
    FEARLUS/SPOM 1-1-4: SPOMSpeciesPatch.h
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/* The SPOMSpeciesPatch object is intended to store all data pertaining to the
 * interaction between a species and a patch in a time-step.
 */

#import <objectbase/SwarmObject.h>

@class SPOMSpecies, SPOMAbstractPatch, SPOMAbstractConnect, SPOMAbstractExtinction,
  SPOMAbstractColonization, SPOMEnvironment;

@interface SPOMSpeciesPatch: SwarmObject {
  SPOMSpecies *species;
  SPOMAbstractPatch *patch;
  SPOMAbstractConnect *connect;
  SPOMAbstractExtinction *extinction;
  SPOMAbstractColonization *colonization;

  double connectivity;
  double seedConnectivity;
  double p_colonization;
  double p_seedColonization;
  double p_extinction;

  double habitat;
  double unsunk_habitat;

  int how_long_there;
  int how_long_not_there;
  BOOL present;
  BOOL prayPresent;
  BOOL areaCellAffected;
  BOOL preOccupancyAffected;
  
  double muPredationModifier;
  int nbPredationMultipliers;
  id <Set> commonPredationHabitatSet;
}

+create: z setSpecies: (SPOMSpecies *)s
                patch: (SPOMAbstractPatch *)p
              connect: (SPOMAbstractConnect *)cnx
           extinction: (SPOMAbstractExtinction *)ext
         colonization: (SPOMAbstractColonization *)cln;
		 
-buildObjects;

-calcConnectivity;
-calcPColonization;
-calcPExtinction;
-calcRescueEffect;
-(double)calcSeedBankEffect;
-updateOccupancy;
-updateCompetitionExclusion;
-updateHabitat;
-outcompeted;
-(void)affectAreaCell;
-(BOOL)getAffectedAreaCell;

-(double)pColonization;
-(double)pExtinction;
-(double)habitat;
-(double)unsunkHabitat;
-(void)setHabitat : (double)val;

-(void)addPredationModifier : (double) coef from: (SPOMSpecies *) predator;
-(double) getPredationModifier;
-(void) resetPredationModifier;
-(double) getCommonPredationHabitat;

-(SPOMSpecies *)species;
-(SPOMAbstractPatch *)patch;
-(int)howLongThere;
-(BOOL)present;
-occupy;
-unoccupy;

@end
