/*
    FEARLUS/SPOM 1-1-4: SubsetSpeciesGovernment.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/* Implementation of SubsetSpeciesGovernment class
 */

#import "SubsetSpeciesGovernment.h"
#import "FearlusOutput.h"
#import "Bug.h"
#import <misc.h>

static int param_n_rewards = 0;
static BOOL given_n_rewards = NO;

@implementation SubsetSpeciesGovernment

/* +writeParameters:
 *
 * Write the number of rewards to give.
 */

+(void)writeParameters: (FILE *)fp {
  if(!given_n_rewards) [Bug file: __FILE__ line: __LINE__];

  fprintf(fp, "Number of rewards:\t%d%s",
	  param_n_rewards, [FearlusOutput nl]);

  [super writeParameters: fp];
}

/* +setParameter:to:
 *
 * Set the budget parameter
 */

+(BOOL)setParameter: (const char *)param to: (const char *)value {
  if(strcmp(param, "MaxNRewards") == 0) {
    param_n_rewards = atoi(value);
    given_n_rewards = YES;
    return YES;
  }
  else return [super setParameter: param to: value];
}

/* -configure
 *
 * Put the parameters in
 */

-configure {
  if(!given_n_rewards) {
    fprintf(stderr, "Error in government file: No MaxNRewards parameter\n");
    abort();
  }

  n_rewards = param_n_rewards;

  if(n_rewards < 0) {
    fprintf(stderr, "Invalid setting for MaxNRewards parameter (%d) in "
	    "government file\n", n_rewards);
    abort();
  }
  return [super configure];
}

/* -administerRewards
 *
 * Administer the rewards, issuing a maximum of n_rewards rewards
 */

-(void)administerRewards {
  [self limitNRewards: n_rewards];
}

@end
