/*
    FEARLUS/SPOM 1-1-4: WorseLUThanMeTrigger.m
    Copyright (C) 2007  Macaulay Institute

    This file is part of FEARLUS/SPOM 1-1-4, an agent-based model of land use
    change and stochastic patch occupancy model.

    FEARLUS/SPOM 1-1-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS/SPOM 1-1-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

/* Implementation of WorseLUThanMeTrigger class
 */

#import "WorseLUThanMeTrigger.h"
#import "AbstractSocialLandManager.h"
#import "LandParcel.h"
#import "LandUse.h"
#import "Bug.h"

@implementation WorseLUThanMeTrigger

/* worseLandUseOf: -> most polluting land use of land manager
 *
 * Compute the most polluting land use of the land manager
 */

-(LandUse *)worstLandUseOf: (AbstractSocialLandManager *)lm {
  double most_pollution;
  id <List> lps = [lm getLandParcels];
  id ix;
  LandParcel *lp;
  LandUse *worstLU;

  ix = [lps begin: scratchZone];
  lp = (LandParcel *)[ix next];
  if([ix getLoc] != Member) [Bug file: __FILE__ line: __LINE__];
				// Land manager should not be checking
				// the pollution of a neighbour with
				// no land parcels
  worstLU = [lp getLandUse];
  most_pollution = [worstLU getPollution];

  for(lp = (LandParcel *)[ix next];
      [ix getLoc] == Member;
      lp = (LandParcel *)[ix next]) {
    LandUse *thisLU;
    double this_pollution;

    thisLU = [lp getLandUse];
    this_pollution = [thisLU getPollution];
    
    if(this_pollution > most_pollution) {
      most_pollution = this_pollution;
      worstLU = thisLU;
    }
  }
  [ix drop];

  return worstLU;
}

/* manager:ofManager:approves:disapproves:
 *
 * Return the approval and disapproval of the object land manager by
 * the subject land manager.
 */

-(void)manager: (AbstractSocialLandManager *)lm_subj
     ofManager: (AbstractSocialLandManager *)lm_obj
      approves: (double *)app
   disapproves: (double *)disapp {
  double subj_worst_pollution = [[self worstLandUseOf: lm_subj] getPollution];
  double obj_worst_pollution = [[self worstLandUseOf: lm_obj] getPollution];

  if(obj_worst_pollution > subj_worst_pollution) (*disapp) = disapproval;
  else (*app) = approval;
}

@end
